package com.plugin.flutter.ipda_scanner;


import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;

import java.util.HashMap;
import java.util.Map;

import android.device.ScanDevice;
import android.os.Bundle;
import android.util.Log;



public class IPdaScannerNative extends IPdaScanner  {

    class Constants
    {

        public static final String ACTION_BARCODE_DATA = "scan.rcv.message";  
        public static final String ACTION_CLAIM_SCANNER = "scan.service.startup";
        public static final String ACTION_RELEASE_SCANNER = "scan.service.close"; 

    }

    class ScanBroadcastReceiver extends BroadcastReceiver
    {
        @Override
        public void onReceive(Context context, Intent intent)
        {
            try
            {

                if ( intent!= null) {
                    Log.i("IPdaScannerNative", "ScanBroadcastReceiver:onReceive" + intent.toString());
                }
                else{

                    Log.i("IPdaScannerNative", "ScanBroadcastReceiver:onReceive [null intent]");
                }
                
                switch(intent.getAction())
                {
                    case Constants.ACTION_BARCODE_DATA:
                        String decoded = decodeIntentBarcode(intent);
                        if ( decoded != null){
                            onDecoded(decoded);
                        }
                        break;
                }
            }catch(Exception e)
            {
                onError(e);
            }
        }
    }

    private boolean initialized, initializing, pendingResume;
    private transient ScanDevice scanner;

    private ScanBroadcastReceiver scanBroadcastReceiver;
    private IntentFilter intentFilter;

    public IPdaScannerNative(Context context)
    {
        super(context);
        pendingResume = false;
        initialized = false;
        initializing = false;
        init();
    }

    
    

    

    private void init()
    {
        initializing = true;

        Log.i("IPdaScannerPlugin", "IPdaScannerNative.init");

        
        
        try{
            Log.i("IPdaScannerPlugin", "IPdaScannerBroadcasts:init");

            scanBroadcastReceiver = new ScanBroadcastReceiver();
            intentFilter = new IntentFilter(Constants.ACTION_BARCODE_DATA);
            
            scanner = new ScanDevice();

            

            scanner.setOutScanMode(0);

            if ( !scanner.isScanOpened()) {
                scanner.openScan();
            }
           
            initialized = true;
            initializing = false;
            if(pendingResume) resumeScanner();
        }

        catch (Exception e){
            onError(e);
        }
    }

    private String decodeIntentBarcode(Intent intent){
        byte[] barocode = intent.getByteArrayExtra("barocode");
        int barocodelen = intent.getIntExtra("length", 0);
        byte temp = intent.getByteExtra("barcodeType", (byte) 0);
        byte[] aimid = intent.getByteArrayExtra("aimid");
        return new String(barocode, 0, barocodelen);
    }
    
    private void registerReceiver()
    {

        try
        {
            context.registerReceiver(scanBroadcastReceiver, intentFilter);
        }catch(Exception e)
        {

            e.getMessage();

            Log.e("IPdaScannerPlugin", "registerReceiver:stopScanner " + e.getMessage());
        }

        try
        {


            Intent intent = new Intent(Constants.ACTION_CLAIM_SCANNER) ;
            context.sendBroadcast(intent);
            
        }catch(Exception e)
        {
            e.getMessage();
        }
    }

    private void unregisterReceiver()
    {
        try
        {
            context.unregisterReceiver(scanBroadcastReceiver);
        }catch(Exception e)
        {
            e.getMessage();
        }

        try
        {
            Intent intent = new Intent(Constants.ACTION_RELEASE_SCANNER);
            context.sendBroadcast(intent);
            
        }catch(Exception e)
        {
            e.getMessage();
        }
    }
  

    private void initProperties(){
        
    }

    @Override

    public void setProperties(Map<String, Object> mapProperties)
    {
        
    }

    @Override
    public boolean resumeScanner()
    {
        startScanner();
        return true;
    }

    @Override
    public boolean pauseScanner()
    {
        if (scanner != null) {
            // release the scanner claim so we don't get any scanner notifications while paused
            // and the scanner properties are restored to default.
            scanner.closeScan();
        }

        unregisterReceiver();

        pendingResume = false;
        return true;
    }

    @Override
    public boolean startScanner()
    {
        Log.i("IPdaScannerPlugin", "IPdaScannerNative:startScanner");

        if (scanner != null){
            try {
                if ( !scanner.isScanOpened()){
                    scanner.openScan();
                }
                
            } catch (Exception e) {
                onError(e);
                return false;
            }
        } else {
            pendingResume = true;
            if(!initialized && !initializing) init();
        }

        registerReceiver();

        return true;
    }

    @Override
    public boolean stopScanner()
    {
        Log.i("IPdaScannerPlugin", "IPdaScannerNative:stopScanner");

        pendingResume = false;
        try
        {
            if (scanner != null) {

                // release the scanner claim so we don't get any scanner notifications while paused
                // and the scanner properties are restored to default.
                 scanner.stopScan();

                // close BarcodeReader to clean up resources.
                scanner.closeScan();
                scanner = null;
            }
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }

        

        unregisterReceiver();
       
        
        initialized = false;
        return true;
    }
}